@@ -29,18 +29,22 @@ type testController struct {
 	kubeClient    *fakekube.Clientset
 }
 
-func newController(work *workapiv1.ManifestWork, mapper *resource.Mapper) *testController {
+func newController(work *workapiv1.ManifestWork, appliedWork *workapiv1.AppliedManifestWork, mapper *resource.Mapper) *testController {
 	fakeWorkClient := fakeworkclient.NewSimpleClientset(work)
 	workInformerFactory := workinformers.NewSharedInformerFactoryWithOptions(fakeWorkClient, 5*time.Minute, workinformers.WithNamespace("cluster1"))
 
 	controller := &ManifestWorkController{
-		manifestWorkClient: fakeWorkClient.WorkV1().ManifestWorks("cluster1"),
-		manifestWorkLister: workInformerFactory.Work().V1().ManifestWorks().Lister().ManifestWorks("cluster1"),
-		restMapper:         mapper,
+		manifestWorkClient:        fakeWorkClient.WorkV1().ManifestWorks("cluster1"),
+		manifestWorkLister:        workInformerFactory.Work().V1().ManifestWorks().Lister().ManifestWorks("cluster1"),
+		appliedManifestWorkClient: fakeWorkClient.WorkV1().AppliedManifestWorks(),
+		appliedManifestWorkLister: workInformerFactory.Work().V1().AppliedManifestWorks().Lister(),
+		restMapper:                mapper,
 	}
 
-	store := workInformerFactory.Work().V1().ManifestWorks().Informer().GetStore()
-	store.Add(work)
+	workInformerFactory.Work().V1().ManifestWorks().Informer().GetStore().Add(work)
+	if appliedWork != nil {
+		workInformerFactory.Work().V1().AppliedManifestWorks().Informer().GetStore().Add(appliedWork)
+	}
 
 	return &testController{
 		controller: controller,
@@ -63,7 +67,7 @@ func (t *testController) withUnstructuredObject(objects ...runtime.Object) *test
 	return t
 }
 
-func assertCondition(t *testing.T, conditions []workapiv1.StatusCondition, expectedCondition string, expectedStatus metav1.ConditionStatus) {
+func assertCondition(t *testing.T, conditions []metav1.Condition, expectedCondition string, expectedStatus metav1.ConditionStatus) {
 	conditionTypeFound := false
 	for _, condition := range conditions {
 		if condition.Type != expectedCondition {
@@ -98,6 +102,7 @@ type testCase struct {
 	spokeDynamicObject         []runtime.Object
 	expectedWorkAction         []string
 	expectedKubeAction         []string
+	expectedAppliedWorkAction  []string
 	expectedDynamicAction      []string
 	expectedManifestConditions []expectedCondition
 	expectedWorkConditions     []expectedCondition
@@ -115,6 +120,7 @@ func newTestCase(name string) *testCase {
 		spokeObject:                []runtime.Object{},
 		spokeDynamicObject:         []runtime.Object{},
 		expectedWorkAction:         []string{},
+		expectedAppliedWorkAction:  []string{},
 		expectedKubeAction:         []string{},
 		expectedDynamicAction:      []string{},
 		expectedManifestConditions: []expectedCondition{},
@@ -142,6 +148,11 @@ func (t *testCase) withExpectedWorkAction(actions ...string) *testCase {
 	return t
 }
 
+func (t *testCase) withAppliedWorkAction(action ...string) *testCase {
+	t.expectedAppliedWorkAction = action
+	return t
+}
+
 func (t *testCase) withExpectedKubeAction(actions ...string) *testCase {
 	t.expectedKubeAction = actions
 	return t
@@ -167,12 +178,27 @@ func (t *testCase) validate(
 	dynamicClient *fakedynamic.FakeDynamicClient,
 	workClient *fakeworkclient.Clientset,
 	kubeClient *fakekube.Clientset) {
-	workActions := workClient.Actions()
-	if len(workActions) != len(t.expectedWorkAction) {
-		ts.Errorf("Expected %d action but got %#v", len(t.expectedWorkAction), workActions)
+	actualWorkActions := []clienttesting.Action{}
+	actualAppliedWorkActions := []clienttesting.Action{}
+	for _, workAction := range workClient.Actions() {
+		if workAction.GetResource().Resource == "manifestworks" {
+			actualWorkActions = append(actualWorkActions, workAction)
+		}
+		if workAction.GetResource().Resource == "appliedmanifestworks" {
+			actualAppliedWorkActions = append(actualAppliedWorkActions, workAction)
+		}
 	}
-	for index := range workActions {
-		spoketesting.AssertAction(ts, workActions[index], t.expectedWorkAction[index])
+	if len(actualWorkActions) != len(t.expectedWorkAction) {
+		ts.Errorf("Expected %d action but got %#v", len(t.expectedWorkAction), actualWorkActions)
+	}
+	for index := range actualWorkActions {
+		spoketesting.AssertAction(ts, actualWorkActions[index], t.expectedWorkAction[index])
+	}
+	if len(actualAppliedWorkActions) != len(t.expectedAppliedWorkAction) {
+		ts.Errorf("Expected %d action but got %#v", len(t.expectedAppliedWorkAction), actualAppliedWorkActions)
+	}
+	for index := range actualAppliedWorkActions {
+		spoketesting.AssertAction(ts, actualAppliedWorkActions[index], t.expectedAppliedWorkAction[index])
 	}
 
 	spokeDynamicActions := dynamicClient.Actions()
@@ -190,7 +216,7 @@ func (t *testCase) validate(
 		spoketesting.AssertAction(ts, spokeKubeActions[index], t.expectedKubeAction[index])
 	}
 
-	actual, ok := workActions[len(workActions)-1].(clienttesting.UpdateActionImpl)
+	actual, ok := actualWorkActions[len(actualWorkActions)-1].(clienttesting.UpdateActionImpl)
 	if !ok {
 		ts.Errorf("Expected to get update action")
 	}
@@ -203,8 +229,8 @@ func (t *testCase) validate(
 	}
 }
 
-func newCondition(name, status, reason, message string, lastTransition *metav1.Time) workapiv1.StatusCondition {
-	ret := workapiv1.StatusCondition{
+func newCondition(name, status, reason, message string, lastTransition *metav1.Time) metav1.Condition {
+	ret := metav1.Condition{
 		Type:    name,
 		Status:  metav1.ConditionStatus(status),
 		Reason:  reason,
@@ -216,7 +242,7 @@ func newCondition(name, status, reason, message string, lastTransition *metav1.T
 	return ret
 }
 
-func newManifestCondition(ordinal int32, resource string, conds ...workapiv1.StatusCondition) workapiv1.ManifestCondition {
+func newManifestCondition(ordinal int32, resource string, conds ...metav1.Condition) workapiv1.ManifestCondition {
 	return workapiv1.ManifestCondition{
 		ResourceMeta: workapiv1.ManifestResourceMeta{Ordinal: ordinal, Resource: resource},
 		Conditions:   conds,
@@ -243,12 +269,14 @@ func TestSync(t *testing.T) {
 		newTestCase("create single resource").
 			withWorkManifest(spoketesting.NewUnstructured("v1", "Secret", "ns1", "test")).
 			withExpectedWorkAction("get", "update").
+			withAppliedWorkAction("create").
 			withExpectedKubeAction("get", "create").
 			withExpectedManifestCondition(expectedCondition{string(workapiv1.ManifestApplied), metav1.ConditionTrue}).
 			withExpectedWorkCondition(expectedCondition{string(workapiv1.WorkApplied), metav1.ConditionTrue}),
 		newTestCase("create single deployment resource").
 			withWorkManifest(spoketesting.NewUnstructured("apps/v1", "Deployment", "ns1", "test")).
 			withExpectedWorkAction("get", "update").
+			withAppliedWorkAction("create").
 			withExpectedDynamicAction("get", "create").
 			withExpectedManifestCondition(expectedCondition{string(workapiv1.ManifestApplied), metav1.ConditionTrue}).
 			withExpectedWorkCondition(expectedCondition{string(workapiv1.WorkApplied), metav1.ConditionTrue}),
@@ -256,12 +284,14 @@ func TestSync(t *testing.T) {
 			withWorkManifest(spoketesting.NewUnstructured("v1", "Secret", "ns1", "test")).
 			withSpokeObject(spoketesting.NewSecret("test", "ns1", "value2")).
 			withExpectedWorkAction("get", "update").
+			withAppliedWorkAction("create").
 			withExpectedKubeAction("get", "delete", "create").
 			withExpectedManifestCondition(expectedCondition{string(workapiv1.ManifestApplied), metav1.ConditionTrue}).
 			withExpectedWorkCondition(expectedCondition{string(workapiv1.WorkApplied), metav1.ConditionTrue}),
 		newTestCase("create single unstructured resource").
 			withWorkManifest(spoketesting.NewUnstructured("v1", "NewObject", "ns1", "test")).
 			withExpectedWorkAction("get", "update").
+			withAppliedWorkAction("create").
 			withExpectedDynamicAction("get", "create").
 			withExpectedManifestCondition(expectedCondition{string(workapiv1.ManifestApplied), metav1.ConditionTrue}).
 			withExpectedWorkCondition(expectedCondition{string(workapiv1.WorkApplied), metav1.ConditionTrue}),
@@ -269,6 +299,7 @@ func TestSync(t *testing.T) {
 			withWorkManifest(spoketesting.NewUnstructuredWithContent("v1", "NewObject", "ns1", "n1", map[string]interface{}{"spec": map[string]interface{}{"key1": "val1"}})).
 			withSpokeDynamicObject(spoketesting.NewUnstructuredWithContent("v1", "NewObject", "ns1", "n1", map[string]interface{}{"spec": map[string]interface{}{"key1": "val2"}})).
 			withExpectedWorkAction("get", "update").
+			withAppliedWorkAction("create").
 			withExpectedDynamicAction("get", "update").
 			withExpectedManifestCondition(expectedCondition{string(workapiv1.ManifestApplied), metav1.ConditionTrue}).
 			withExpectedWorkCondition(expectedCondition{string(workapiv1.WorkApplied), metav1.ConditionTrue}),
@@ -276,6 +307,7 @@ func TestSync(t *testing.T) {
 			withWorkManifest(spoketesting.NewUnstructured("v1", "Secret", "ns1", "test"), spoketesting.NewUnstructured("v1", "Secret", "ns2", "test")).
 			withSpokeObject(spoketesting.NewSecret("test", "ns1", "value2")).
 			withExpectedWorkAction("get", "update").
+			withAppliedWorkAction("create").
 			withExpectedKubeAction("get", "delete", "create", "get", "create").
 			withExpectedManifestCondition(expectedCondition{string(workapiv1.ManifestApplied), metav1.ConditionTrue}, expectedCondition{string(workapiv1.ManifestApplied), metav1.ConditionTrue}).
 			withExpectedWorkCondition(expectedCondition{string(workapiv1.WorkApplied), metav1.ConditionTrue}),
@@ -285,7 +317,7 @@ func TestSync(t *testing.T) {
 		t.Run(c.name, func(t *testing.T) {
 			work, workKey := spoketesting.NewManifestWork(0, c.workManifest...)
 			work.Finalizers = []string{controllers.ManifestWorkFinalizer}
-			controller := newController(work, spoketesting.NewFakeRestMapper()).
+			controller := newController(work, nil, spoketesting.NewFakeRestMapper()).
 				withKubeObject(c.spokeObject...).
 				withUnstructuredObject(c.spokeDynamicObject...)
 			syncContext := spoketesting.NewFakeSyncContext(t, workKey)
@@ -305,17 +337,17 @@ func TestFailedToApplyResource(t *testing.T) {
 		withWorkManifest(spoketesting.NewUnstructured("v1", "Secret", "ns1", "test"), spoketesting.NewUnstructured("v1", "Secret", "ns2", "test")).
 		withSpokeObject(spoketesting.NewSecret("test", "ns1", "value2")).
 		withExpectedWorkAction("get", "update").
+		withAppliedWorkAction("create").
 		withExpectedKubeAction("get", "delete", "create", "get", "create").
 		withExpectedManifestCondition(expectedCondition{string(workapiv1.ManifestApplied), metav1.ConditionTrue}, expectedCondition{string(workapiv1.ManifestApplied), metav1.ConditionFalse}).
 		withExpectedWorkCondition(expectedCondition{string(workapiv1.WorkApplied), metav1.ConditionFalse})
 
 	work, workKey := spoketesting.NewManifestWork(0, tc.workManifest...)
 	work.Finalizers = []string{controllers.ManifestWorkFinalizer}
-	controller := newController(work, spoketesting.NewFakeRestMapper()).withKubeObject(tc.spokeObject...).withUnstructuredObject()
+	controller := newController(work, nil, spoketesting.NewFakeRestMapper()).withKubeObject(tc.spokeObject...).withUnstructuredObject()
 
 	// Add a reactor on fake client to throw error when creating secret on namespace ns2
 	controller.kubeClient.PrependReactor("create", "secrets", func(action clienttesting.Action) (handled bool, ret runtime.Object, err error) {
-		fmt.Printf("the action get into %v\n", action)
 		if action.GetVerb() != "create" {
 			return false, nil, nil
 		}
@@ -392,14 +424,14 @@ func TestGenerateUpdateStatusFunc(t *testing.T) {
 
 	cases := []struct {
 		name                     string
-		startingStatusConditions []workapiv1.StatusCondition
+		startingStatusConditions []metav1.Condition
 		manifestConditions       []workapiv1.ManifestCondition
-		expectedStatusConditions []workapiv1.StatusCondition
+		expectedStatusConditions []metav1.Condition
 	}{
 		{
 			name:                     "no manifest condition exists",
 			manifestConditions:       []workapiv1.ManifestCondition{},
-			expectedStatusConditions: []workapiv1.StatusCondition{},
+			expectedStatusConditions: []metav1.Condition{},
 		},
 		{
 			name: "all manifests are applied successfully",
@@ -407,7 +439,7 @@ func TestGenerateUpdateStatusFunc(t *testing.T) {
 				newManifestCondition(0, "resource0", newCondition(string(workapiv1.ManifestApplied), string(metav1.ConditionTrue), "my-reason", "my-message", nil)),
 				newManifestCondition(1, "resource1", newCondition(string(workapiv1.ManifestApplied), string(metav1.ConditionTrue), "my-reason", "my-message", nil)),
 			},
-			expectedStatusConditions: []workapiv1.StatusCondition{
+			expectedStatusConditions: []metav1.Condition{
 				newCondition(string(workapiv1.WorkApplied), string(metav1.ConditionTrue), "AppliedManifestWorkComplete", "Apply manifest work complete", nil),
 			},
 		},
@@ -417,33 +449,33 @@ func TestGenerateUpdateStatusFunc(t *testing.T) {
 				newManifestCondition(0, "resource0", newCondition(string(workapiv1.ManifestApplied), string(metav1.ConditionTrue), "my-reason", "my-message", nil)),
 				newManifestCondition(1, "resource1", newCondition(string(workapiv1.ManifestApplied), string(metav1.ConditionFalse), "my-reason", "my-message", nil)),
 			},
-			expectedStatusConditions: []workapiv1.StatusCondition{
+			expectedStatusConditions: []metav1.Condition{
 				newCondition(string(workapiv1.WorkApplied), string(metav1.ConditionFalse), "AppliedManifestWorkFailed", "Failed to apply manifest work", nil),
 			},
 		},
 		{
 			name: "update existing status condition",
-			startingStatusConditions: []workapiv1.StatusCondition{
+			startingStatusConditions: []metav1.Condition{
 				newCondition(string(workapiv1.WorkApplied), string(metav1.ConditionTrue), "AppliedManifestWorkComplete", "Apply manifest work complete", &transitionTime),
 			},
 			manifestConditions: []workapiv1.ManifestCondition{
 				newManifestCondition(0, "resource0", newCondition(string(workapiv1.ManifestApplied), string(metav1.ConditionTrue), "my-reason", "my-message", nil)),
 				newManifestCondition(1, "resource1", newCondition(string(workapiv1.ManifestApplied), string(metav1.ConditionTrue), "my-reason", "my-message", nil)),
 			},
-			expectedStatusConditions: []workapiv1.StatusCondition{
+			expectedStatusConditions: []metav1.Condition{
 				newCondition(string(workapiv1.WorkApplied), string(metav1.ConditionTrue), "AppliedManifestWorkComplete", "Apply manifest work complete", &transitionTime),
 			},
 		},
 		{
 			name: "override existing status conditions",
-			startingStatusConditions: []workapiv1.StatusCondition{
+			startingStatusConditions: []metav1.Condition{
 				newCondition(string(workapiv1.WorkApplied), string(metav1.ConditionTrue), "AppliedManifestWorkComplete", "Apply manifest work complete", nil),
 			},
 			manifestConditions: []workapiv1.ManifestCondition{
 				newManifestCondition(0, "resource0", newCondition(string(workapiv1.ManifestApplied), string(metav1.ConditionTrue), "my-reason", "my-message", nil)),
 				newManifestCondition(1, "resource1", newCondition(string(workapiv1.ManifestApplied), string(metav1.ConditionFalse), "my-reason", "my-message", nil)),
 			},
-			expectedStatusConditions: []workapiv1.StatusCondition{
+			expectedStatusConditions: []metav1.Condition{
 				newCondition(string(workapiv1.WorkApplied), string(metav1.ConditionFalse), "AppliedManifestWorkFailed", "Failed to apply manifest work", nil),
 			},
 		},
@@ -452,7 +484,7 @@ func TestGenerateUpdateStatusFunc(t *testing.T) {
 	controller := &ManifestWorkController{}
 	for _, c := range cases {
 		t.Run(c.name, func(t *testing.T) {
-			updateStatusFunc := controller.generateUpdateStatusFunc(workapiv1.ManifestResourceStatus{Manifests: c.manifestConditions})
+			updateStatusFunc := controller.generateUpdateStatusFunc(c.manifestConditions)
 			manifestWorkStatus := &workapiv1.ManifestWorkStatus{
 				Conditions: c.startingStatusConditions,
 			}
@@ -529,7 +561,7 @@ func TestAllInCondition(t *testing.T) {
 	}
 }
 
-func TestBuildManifestResourceMeta(t *testing.T) {
+func TestBuildResourceMeta(t *testing.T) {
 	var secret *corev1.Secret
 	var u *unstructured.Unstructured
 
@@ -580,7 +612,48 @@ func TestBuildManifestResourceMeta(t *testing.T) {
 
 	for _, c := range cases {
 		t.Run(c.name, func(t *testing.T) {
-			actual, err := buildManifestResourceMeta(0, c.object, c.restMapper)
+			actual, err := buildResourceMeta(0, c.object, c.restMapper)
+			if err != nil {
+				t.Errorf("Should be success with no err: %v", err)
+			}
+
+			actual.Ordinal = c.expected.Ordinal
+			if !equality.Semantic.DeepEqual(actual, c.expected) {
+				t.Errorf(diff.ObjectDiff(actual, c.expected))
+			}
+		})
+	}
+}
+
+func TestBuildManifestResourceMeta(t *testing.T) {
+	cases := []struct {
+		name           string
+		applyResult    runtime.Object
+		manifestObject runtime.Object
+		restMapper     *resource.Mapper
+		expected       workapiv1.ManifestResourceMeta
+	}{
+		{
+			name:        "extract meta from apply result",
+			applyResult: spoketesting.NewSecret("test1", "ns1", "value1"),
+			restMapper:  spoketesting.NewFakeRestMapper(),
+			expected:    workapiv1.ManifestResourceMeta{Version: "v1", Kind: "Secret", Resource: "secrets", Namespace: "ns1", Name: "test1"},
+		},
+		{
+			name:           "fall back to manifest",
+			manifestObject: spoketesting.NewSecret("test2", "ns2", "value2"),
+			restMapper:     spoketesting.NewFakeRestMapper(),
+			expected:       workapiv1.ManifestResourceMeta{Version: "v1", Kind: "Secret", Resource: "secrets", Namespace: "ns2", Name: "test2"},
+		},
+	}
+
+	for _, c := range cases {
+		t.Run(c.name, func(t *testing.T) {
+			manifest := workapiv1.Manifest{}
+			if c.manifestObject != nil {
+				manifest.Object = c.manifestObject
+			}
+			actual, err := buildManifestResourceMeta(0, c.applyResult, manifest, c.restMapper)
 			if err != nil {
 				t.Errorf("Should be success with no err: %v", err)
 			}
